#!/bin/vtcl
# -----------------------------------------------------------------------------
# Copyright 1994 by SCO, Inc.
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that 
# the above copyright notice appear in all copies and that both that copyright
# notice  and  this  permission  notice  appear in  supporting  documentation,
# and  that  the name  of  SCO  not  be used  in advertising or publicity 
# pertaining  to distribution of  the  software without  specific,  written 
# prior permission.   SCO makes no representations  about  the  suitability of
# this  software for any purpose.  It is provided "as is" without express or 
# implied warranty.
# -----------------------------------------------------------------------------
# Demo		: tclLock
# Description	: SCO Lock Configuration Interface
# Comments	: Example of how to make it easy to embellish a 
#		  basic utility.  This script uses keyed lists to squirrel
#		  data away.  It could just as easily use associative arrays
#		  (using the keys as indexes), EXCEPT, that the keyedlist
#		  is easier to save as one string to a file, then read in
#		  again (in order to preserve state between scolock.tcl 
#		  sessions).
# Things to 
# Look for	- use of "env" to retrieve shell environmental information.
# 		- use of format command
#		- use of "-defaultButton" to set initial form focus.

proc OkCB {cbs} { VtClose; exit 0 }

proc SetMsgCB {cbs} {
	global globalData
	set dlog [keylget cbs dialog]
	set tf [WxGetVar $dlog tf]
	set defstring [VtGetValues $tf -value]
	set mainF $globalData(MAINDIALOG)
	set messageTextW [WxGetVar $mainF messageTextW]
	VtSetValues $messageTextW -value $defstring
	set globalData(DEFAULTMESSAGE) $defstring
	VtDestroyDialog $dlog
}
#------------------------------------------------------------------
# Set the default string that the user would like to have come
# up by default if they just select OK.
#
proc SetDefaultGreeting {cbs} {
	set dlog [keylget cbs dialog]
	set form [VtFormDialog $dlog.form \
		-title "Set Default Greeting" \
		-apply \
		-applyCallback SetMsgCB \
		-cancelCallback {WxEndFormCB} \
		]

	set lbl [VtLabel $form.label \
		-label "Enter your default greeting..." \
		]
	set tf [VtText $form.tf \
		-rows 4 \
		-columns 40 \
		-rightSide FORM \
		-bottomSide FORM \
		]
	WxSetVar $form tf $tf
	VtShow $form
	VtSetFocus $tf
}

#------------------------------------------------------------------
# retrieve all values saved away in .tclLock.
#
proc GetAttrValuesFromFile {} {
	global globalData env

	if { [info exists env(HOME)] } {
		set filename $env(HOME)/.tclLock
	} else {
		set msg "Unable to get your home directory name" 
			set mainF $globalData(MAINDIALOG)
			VtShow [VtErrorDialog $mainF.errorDialog \
				-message $msg \
			]
			VtMainLoop
	}

	if { ! [file exists $filename] } {
		return
	}
	set fd [open $filename r ]
	
	set recordCount [lgets $fd line] 
	if { $recordCount < 1 } {
		set msg "Problem with $filename format" 
		set mainF $globalData(MAINDIALOG)
		VtShow [VtErrorDialog $mainF.errorDialog \
			-message $msg \
			]
	}

	# Build the global associative array from the keyed list.
	#
	foreach key_value [keylkeys line] {
		set globalData($key_value) [keylget line $key_value]
	}
	close $fd
}

# Save keyed list to file.
#
proc SaveAttrValuesToFile {} {
	global globalData env
		set fd [open $env(HOME)/.tclLock w]

		# 1st, build a temp keyed list
		foreach key_value [array names globalData] {
			keylset tmpKeyedList $key_value $globalData($key_value)
		}
		# 2nd, blast the keyed list into a file.
		puts $fd $tmpKeyedList
		close $fd
}

#------------------------------------------------------------------
# SetPixmapCB ()
#	- Needs to be updated to remember the chosen pixmap past 
#	  the current tclLock session.
#
proc SetPixmapCB {cbs} {
global pixmap 
	set dlog [keylget cbs dialog]
	set pixmapDirectory "/usr/include/X11/bitmaps/*"
	set selbox [VtFileSelectionDialog $dlog.selbox \
		-title "Pixmap for ScoLock Background" \
		-filter $pixmapDirectory \
		-okCallback SetPixmapLblCB \
		]
	VtShow $selbox
}
proc SetPixmapLblCB {cbs} {
	global globalData 
	set globalData(PIXMAP) [keylget cbs selection]
	VtSetValues $globalData(PIXMAPLABELW) \
		-label "Current pixmap is: [file tail $globalData(PIXMAP)]" 
}

#---------
# SetItCB()
#	- grab the text from the textfield, imbed it with cntl-n's
#	  where necessary (so they'll map to the labelfield widget).
#	- get the current time to stuff into the labelfield message.
#
proc SetItCB {cbs} {
	global globalData
	set dlog [keylget cbs dialog]
	set messageTextW [WxGetVar $dlog messageTextW]
	set string [VtGetValues $messageTextW -value]
	if [lempty string] {
		OkCB dummy
	}
	# replace \n with \\n and put result in variable: temp
	regsub -all \n $string \\n temp

	# Get the date stamp
	set clockValue [getclock]
	set dateToday [fmtclock $clockValue "%A, %d %b %y @ %r"]

	set newString [format "%s%s%s" $dateToday\\n ===\\n $temp]
	
	SaveAttrValuesToFile

	catch {
	exec scolock \
		-xrm "scolock*innerForm.backgroundPixmap: $globalData(PIXMAP)" \
		-xrm "scolock*prompt.fontList: 		12x24" \
		-xrm "scolock*prompt.shadowThickness: 	12" \
		-xrm "scolock*prompt.borderWidth: 	6" \
		-xrm "scolock*prompt.foreground: 	blue" \
		-xrm "scolock*prompt.background: 	pink" \
		-xrm "scolock*prompt.borderColor: 	red" \
		-xrm "scolock*prompt.labelString: 	$newString" &
	} errorMessage
	OkCB dummy
}

proc ClearText {cbs} {
	set dlog [keylget cbs dialog]
	set messageTextW [WxGetVar $dlog messageTextW] 
	VtSetValues $messageTextW -value {}
	VtSetFocus $messageTextW
	#VtSetValue msglabel -value "Enter some text..."
}

proc CheckForScoLockBinary  {appShell} {
	if { ! [file exists /usr/bin/X11/scolock] } {
		set msg "Lock Screen Customizer\n" 
		append msg "Unable to located /usr/bin/scolock." 
		VtShow [VtErrorDialog $appShell.fred \
			-ok \
			-okLabel Exit \
			-okCallback OkCB \
			-message $msg \
			]
		# Got to put the mainloop here since we haven't entered
		# the program's main loop at the bottom of the script.
		VtMainLoop
	}
}
proc PostCharmInfo {parent} {
	VtShow [VtInformationDialog $parent.ibox \
		-message "Because scolock only works in the X/Motif\n \
			  environment, this demo is only useful in \n \
			  the X/Motif environment." \
		-ok \
		-okCallback OkCB \
		]
	VtMainLoop
}

# Global Data Array
#
set globalData(MAINDIALOG) {}
set globalData(PIXMAP) "<Unselected>"
set globalData(PIXMAPLABELW) {}
set globalData(DEFAULTMESSAGE) "I've locked this display
Have a nice day."
if { [info exists env(USER)] } {
	append globalData(DEFAULTMESSAGE)  " - $env(USER)"
}

#---------
# main()
#
set appShell [VtOpen tclLock]

if { [VtInfo -charm] } {
	PostCharmInfo $appShell
}

CheckForScoLockBinary $appShell

set mainF [VtFormDialog $appShell.mainF \
	-title "Lock Screen Customizer" \
	-okLabel "Lock!" \
	-okCallback SetItCB \
	-cancelCallback OkCB \
	-defaultButton "OK" \
	]

set globalData(MAINDIALOG) $mainF

GetAttrValuesFromFile

set frame [VtFrame $mainF.frame \
	-topSide FORM \
	-shadowType ETCHED_IN \
	]
set clearPushB [VtPushButton $frame.clearPushB \
	-label "Create\nNew\nMessage" \
	-labelCenter \
	-callback ClearText \
	]

set rc [VtRowColumn $mainF.rc \
	-vertical \
	-spacing 0 \
	-numColumns 1 \
	-leftSide NONE \
	-topSide FORM \
	-rightSide FORM \
	]
set setPixPushB [VtPushButton $rc.setPixPushB \
	-label "Set Pixmap" \
	-labelCenter \
	-callback SetPixmapCB \
	]

set setGreetingPushB [VtPushButton $rc.setGreetingPushB \
	-label "Default Greeting" \
	-labelCenter \
	-callback SetDefaultGreeting \
	]

set messageTextW [VtText $mainF.messageTextW \
	-rows 4 \
	-columns 40 \
	-value $globalData(DEFAULTMESSAGE) \
	-rightSide FORM \
	-topSide $frame \
	]

# Squirrel away the Text Widget name for use later on.
WxSetVar $mainF messageTextW $messageTextW

set pixFile [file tail $globalData(PIXMAP)]
set pixL [VtLabel $mainF.pixL \
	-topSide $messageTextW \
	-leftSide FORM \
	-bottomSide FORM \
	-label "Current pixmap: $pixFile" \
	]

set globalData(PIXMAPLABELW) $pixL

VtShow $mainF
VtMainLoop
